@@ -1,5 +1,4 @@
 import random
-import feather
 import numpy as np
 import torch
 import torchvision.transforms as transforms
@@ -71,36 +70,35 @@ class DataLoader():
 
 
 ################################################################################################################################
-#  Dataset, Data
+# Dataset, Data
+#
+# Classification: y type is list or numpy long
+# Mask of integers: 2D classification
+# Mask of floats: ???
+# Regression: y type is float scalar
+#
 ################################################################################################################################
 
 class Dataset():
-    def __init__(self, x, y, clsf=False, cls_to_idx=None, regr=False, trfs=[]):
-        self.x, self.y, self.clsf, self.regr, self.trfs = x, y, clsf, regr, trfs or []
+    def __init__(self, x, y, aug_size=None, clsf=False, cls_to_idx=None, cls=None, regr=False, trfs=[]):
+        self.x,self.y,self.aug_size,self.clsf,self.regr,self.trfs = x,y,aug_size,clsf,regr,trfs or []
         self.bps = None
-        self.cls, self.c, self.cls_to_idx, self.c_idxs, self.c_szs, self.c_dist = None, None, None, None, None, None
         if clsf:
-            assert type(y)==list
-            self.labels = [str(e) for e in y]
-            if cls_to_idx is None:
-                self.cls = sorted(list(set(self.labels)))
-                self.cls_to_idx = {self.cls[i]: i for i in range(len(self.cls))}
-            else:
-                self.cls = [e for e in cls_to_idx]
-                self.cls_to_idx = cls_to_idx
-                for label in self.labels:
-                    if label not in self.cls:
-                        print(f'Label {label} not found in cls_to_idx')
-                        raise Exception()
-            self.c = len(self.cls)
-            self.y = torch.tensor([self.cls_to_idx[label] for label in self.labels])
-            self.c_idxs, self.c_szs, self.c_dist = get_cls_dist(self.labels, self.cls_to_idx)
-            print(f'\n{self.__class__.__name__} for Classification: size: {len(self.y)}, cls: {self.c}')
-            show_dist(self.labels)
-    def __len__(self): return len(self.x)
+            y,self.cls,self.c,self.labels,self.cls_to_idx,self.c_idxs,self.c_szs,self.c_dist = prep_cls(y, cls_to_idx=cls_to_idx, cls=cls)
+            self.y = y.astype(np.int32)
+            show_clsf_stats(self.y, self.c, self.cls, self.c_szs, self.c_dist)
+        else:
+            self.y = y.astype(np.float32)
+        self.x = x #.astype(np.float64)
+        
+    def __len__(self): return self.aug_size or len(self.x)
     def __getitem__(self, i):
+        if self.aug_size: i=random.randint(0,len(self.x)-1)
         x,y = self.load(self.x[i], self.y[i])
-        for trf in self.trfs: x = trf(x)
+        for trf in self.trfs: x,y=trf(x,y)
+        x = torch.from_numpy(x).float()
+        y = torch.from_numpy(y) if len(y.shape)>0 else torch.tensor(y).float()
+        if self.clsf: y = y.long()
         return x,y
     def load(self, x, y): return x,y
     def show(self, idxs, labels=None):
@@ -113,9 +111,9 @@ class Dataset():
 
 
 class Data:
-    def __init__(self, ds1, ds2, clsf=False, regr=False, bs=64):
+    def __init__(self, ds1, ds2, bs=64):
         self.ds1, self.ds2 =  ds1, ds2
-        self.clsf, self.regr, self.bs = clsf, regr, bs
+        self.bs = bs
         # w, pin = 0, torch.cuda.is_available()
         self.dl1 = DataLoader(ds1, bs=bs, shuffle=True)
         self.dl2 = DataLoader(ds2, bs=bs, shuffle=False) if ds2 is not None else None
@@ -124,6 +122,10 @@ class Data:
     def show_batch_2(self, n=8): self.ds2.show(next(iter(self.dl2.sampler))[:n])
     def show_trfs(self, n=8): self.ds1.show_trfs(n=n)
 
+    @property
+    def clsf(self): return self.ds1.clsf
+    @property
+    def regr(self): return self.ds1.regr
     @property
     def c(self): return self.ds1.c if self.clsf else None
     @property
@@ -131,6 +133,39 @@ class Data:
     @property
     def cls_to_idx(self): return self.ds1.cls_to_idx if self.clsf else None
 
+    @classmethod
+    def from_numpy(cls, x1, y1, x2, y2, aug_size1=None, aug_size2=None, clsf=False, regr=False, trfs1=[], trfs2=[], bs=64):
+        ds1 = Dataset(x1, y1,aug_size=aug_size1, clsf=clsf, regr=regr, trfs=trfs1)
+        ds2 = Dataset(x2, y2, aug_size=aug_size2, clsf=clsf, regr=regr, trfs=trfs2)
+        return cls(ds1, ds2, bs=bs)
+
+################################################################################################################################
+#  SignalDataset, SignalData
+################################################################################################################################
+
+class SignalDataset(Dataset):    
+    def __init__(self, *args, **kwargs):
+        super(SignalDataset, self).__init__(*args, **kwargs)
+    def show(self, idxs, labels=None):
+        idxs, labels = listify(idxs), listify(labels)
+        for i,idx in enumerate(idxs):
+            x,y = self.decode(*self[idx])
+            if self.clsf: y = self.cls[y]
+            label = labels[i] if len(labels) else y
+            sigshow(x, label or y)
+
+class SignalData(Data):
+    def __init__(self, x1, y1, x2, y2, clsf=False, regr=False, trfs1=[], trfs2=[], bs=64):
+        ds1 = SignalDataset(x1, y1, clsf=clsf, regr=regr, trfs=trfs1)
+        ds2 = SignalDataset(x2, y2, clsf=clsf, regr=regr, trfs=trfs2)
+        super(SignalData, self).__init__(ds1, ds2, bs=bs)
+        
+    @classmethod
+    def from_numpy(cls, x1, y1, x2, y2, clsf=False, regr=False, trfs1=[], trfs2=[], bs=64):
+        # x1,x2 = torch.from_numpy(x1).float(),torch.from_numpy(x2).float()
+        # if clsf: y1, y2 = list(y1), list(y2)
+        # else: y1, y2 = torch.from_numpy(y1).float(), torch.from_numpy(y2).float()
+        return cls(x1, y1, x2, y2, clsf=clsf, regr=regr, trfs1=trfs1, trfs2=trfs2, bs=bs)
 
 ################################################################################################################################
 #  ImageDataset, ImageDatasetMem, ImageData
@@ -174,33 +209,6 @@ class ImageData(Data):
         x1, y1, x2, y2 = T(x1).float(), y1, T(x2).float(), y2
         return cls(x1, y1, x2, y2, clsf=clsf, regr=regr, trfs1=trfs1, trfs2=trfs2, bs=bs, ds_class=ImageDatasetMem)
 
-################################################################################################################################
-#  SignalDataset, SignalData
-################################################################################################################################
-
-class SignalDataset(Dataset):    
-    def show(self, idxs, labels=None):
-        idxs, labels = listify(idxs), listify(labels)
-        for i,idx in enumerate(idxs):
-            x,y = self.decode(*self[idx])
-            if self.clsf: y = self.cls[y]
-            label = labels[i] if len(labels) else y
-            sigshow(x, label or y)
-
-class SignalData(Data):
-    def __init__(self, x1, y1, x2, y2, clsf=False, regr=False, trfs1=[], trfs2=[], bs=64):
-        ds1 = SignalDataset(x1, y1, clsf=clsf, regr=regr, trfs=trfs1)
-        ds2 = SignalDataset(x2, y2, clsf=clsf, regr=regr, trfs=trfs2)
-        super(SignalData, self).__init__(ds1, ds2, clsf=clsf, regr=regr, bs=bs)
-        
-    @classmethod
-    def from_numpy(cls, x1, y1, x2, y2, clsf=False, regr=False, trfs1=[], trfs2=[], bs=64):
-        x1, x2 = torch.from_numpy(x1).float(), torch.from_numpy(x2).float()
-        if clsf: y1, y2 = list(y1), list(y2)
-        else: y1, y2 = torch.from_numpy(y1).float(), torch.from_numpy(y2).float()
-        return cls(x1, y1, x2, y2, clsf=clsf, regr=regr, trfs1=trfs1, trfs2=trfs2, bs=bs)
-
-
 ################################################################################################################################
 #   TabularDataset, TabularData
 ################################################################################################################################
