@@ -1,15 +1,9 @@
-import os
-import numpy as np
 import torch
-from IPython.display import display, clear_output
-from dl2050utils.core import listify, camel2snake
 from dl2050nn.cbs import CancelBatchException, CancelEpochException, CancelTrainException, LR_Find, Recorder
 from dl2050nn.optimizers import *
 from dl2050nn.log import LoggerCallback
-from dl2050nn.metrics import accuracy
 from dl2050nn.etc import *
 
-
 class Learner():
 
     def __init__(self, data, model, opt=Adam, loss_f=None, metrics=[], cbs=[], device=None, name=None, path='.', save=True, load=False, plot=True):
