@@ -2,8 +2,13 @@ import math
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from dl2050nn.etc import get_model_n_params, get_cnn_szs
+from dl2050nn.etc import get_cnn_szs
 
+def model_stats(model, name='Model', szs=None):
+    p = sum([p.numel() for p in model.parameters()])
+    s = f'{name}: params: {p/1e6:,.1f}m'
+    if szs: s+=f'last conv: {szs[-1]}'
+    print(s)
 
 def init_layer(l):
     if getattr(l, 'bias', None) is not None: nn.init.constant_(l.bias, 0) # l.bias.data.zero_()
@@ -14,28 +19,22 @@ def init_layer(l):
         #w.uniform_(-sc, sc)
     if isinstance(l, (nn.Conv1d, nn.Conv2d)): nn.init.kaiming_normal_(l.weight)
 
-
 def init_net(m):
     init_layer(m)
     for l in m.children(): init_net(l)
 
-
 def noop(x): return x
     
-
 # def ReLU_Sim(x): return x.clamp_min(0.) - 0.5
 class ReLU_Sim(nn.Module):
     def forward(self, x): return x.clamp_min(0.) - 0.5
 
-
 class Mish(nn.Module):
     def __init__(self): super().__init__()
     def forward(self, x): return x *( torch.tanh(F.softplus(x)))
 
-
 default_act =  Mish() # nn.ReLU(inplace=True), Mish()
 
-
 class Sigmoid(nn.Module):
     def __init__(self, yrange=None):
         super().__init__()
@@ -47,12 +46,10 @@ class Sigmoid(nn.Module):
 #     "Same as `torch.sigmoid`, plus clamping to `(eps,1-eps)"
 #     return input.sigmoid().clamp(eps,1-eps)
 
-
 class Flatten(nn.Module):
     def forward(self, x): return x.view(x.size(0), -1)
 
-
-def Linear(n1, n2, p=0.0, bn=True, act=default_act):
+def Linear(n1, n2, p=None, bn=True, act=default_act):
     net = []
     if bn:
         batch = nn.BatchNorm1d(num_features=n1)
@@ -62,12 +59,11 @@ def Linear(n1, n2, p=0.0, bn=True, act=default_act):
     linear = nn.Linear(in_features=n1, out_features=n2)
     init_layer(linear)
     net += [linear]
-    if act: net += [act]
+    if act: net+=[act]
     return nn.Sequential(*net)
 
 def LinearFinal(n1, n2, p=None, act=nn.LogSoftmax(dim=1)): return Linear(n1, n2, p=p, bn=False, act=act)
 
-
 def Conv2D(n1, n2, ks=3, stride=1, zero_bn=False, act=True):
     bn = nn.BatchNorm2d(n1)
     nn.init.constant_(bn.weight, 0. if zero_bn else 1.)
@@ -77,7 +73,6 @@ def Conv2D(n1, n2, ks=3, stride=1, zero_bn=False, act=True):
     return nn.Sequential(*layers)
     # PREVIOUS: return F.relu(self.batch(self.conv(x)))
 
-
 def Conv1D(n1, n2, ks=3, stride=1, zero_bn=False, act=True):
     bn = nn.BatchNorm1d(n1)
     nn.init.constant_(bn.weight, 0. if zero_bn else 1.)
@@ -86,7 +81,6 @@ def Conv1D(n1, n2, ks=3, stride=1, zero_bn=False, act=True):
     if act: layers.append(default_act)
     return nn.Sequential(*layers)
 
-
 class ResBlock2D(nn.Module):
     def __init__(self, ni, nf, stride=1):
         super().__init__()
@@ -95,21 +89,84 @@ class ResBlock2D(nn.Module):
         self.convs = nn.Sequential(*layers)
         self.idconv = noop if ni==nf else Conv2D(ni, nf, ks=1, act=False)
         self.pool = noop if stride==1 else nn.AvgPool2d(2, ceil_mode=True)
-    
     def forward(self, x): return default_act(self.convs(x) + self.idconv(self.pool(x)))
 
-
 class ResBlock1D(nn.Module):
-    def __init__(self, ni, nf, ks=3, stride=1):
+    def __init__(self, n1, n2, ks=3, stride=1):
         super().__init__()
-        layers  = [Conv1D(ni, nf, ks=ks, stride=stride),
-                   Conv1D(nf, nf, ks=ks, zero_bn=True, act=False)]
+        layers  = [Conv1D(n1, n2, ks=ks, stride=stride),
+                   Conv1D(n2, n2, ks=ks, zero_bn=True, act=False)]
         self.convs = nn.Sequential(*layers)
-        self.idconv = noop if ni==nf else Conv1D(ni, nf, ks=1, act=False)
+        self.idconv = noop if n1==n2 else Conv1D(n1, n2, ks=1, act=False)
         self.pool = noop if stride==1 else nn.AvgPool1d(2, ceil_mode=True)
-
     def forward(self, x): return default_act(self.convs(x) + self.idconv(self.pool(x)))
 
+class Resnet(nn.Module):
+    def __init__(self, data, layers=[64, 64, 64], p=None):
+        super(Resnet, self).__init__()
+        sz = [int(e) for e in data.ds1[0][0].shape]
+        n1, n2 = sz[0], 32
+        net = [ResBlock2D(n1, n2, stride=2)]
+        n1 = n2
+        for n2 in layers:
+            net += [ResBlock2D(n1, n2, stride=2)]
+            n1 = n2
+        net += [nn.AdaptiveMaxPool2d(1)]
+        net += [Flatten()]
+        net += [Linear(layers[-1], 1024, p=p)]
+        net += [LinearFinal(1024, data.c, p=None)]
+        self.net = nn.Sequential(*net)
+        p = sum([p.numel() for p in self.parameters()])
+        self.szs = get_cnn_szs(self, sz)
+        print(f'\nResnet: params: {p:,.0f}, img_sz: {sz}, last_conv {self.szs[-1]}: ')
+
+    def forward(self, x): return self.net(x)
+
+################################################################################################################################
+#  SignalNet
+################################################################################################################################
+
+class SignalNet(nn.Module):
+    def __init__(self, data, p=0.):
+        super(SignalNet, self).__init__()
+        x,_ = data.ds1[0]
+        n1,sz = x.shape[0],x.shape[1]
+        n2 = min((n1+1)*8,64)
+        net = []
+        szs = [(n1,sz)]
+        for i in range(3):
+            net += [Conv1D(n1, n2, stride=2)]
+            n1, sz = n2, int(sz/2)
+            szs += [(n1,sz)]
+        bottle = int((int(math.log2(sz)))/2)
+        for i in range(bottle):
+            net += [ResBlock1D(n1, n1*2, ks=3, stride=2)]
+            n1, sz = n1*2, int(sz/2)
+            szs += [(n1,sz)]
+            net += [nn.MaxPool1d(2)]  # AvgPool1d, MaxPool1d
+            sz = int(sz/2)
+            szs += [(n1,sz)]
+        net += [nn.AdaptiveMaxPool1d(1)]
+        net += [Flatten()]
+        net += [Linear(1*n1, 1024, p=p)]
+        net += [LinearFinal(1024, data.c, p=None)]
+        self.net = nn.Sequential(*net)
+        p = sum([p.numel() for p in self.parameters()])
+        print(f'SignalNet: params: {p:,.0f}, signal_sz: {szs[0]}, bottles: {bottle}, last_conv {szs[-1]}')
+        # print(szs)
+
+    def forward(self, x): return self.net(x)
+
+################################################################################################################################
+#  UNet1D
+################################################################################################################################
+
+class UNet1D(nn.Module):
+    pass
+
+################################################################################################################################
+#  TabularNet
+################################################################################################################################
 
 class TabularNet(nn.Module):
     def __init__(self, data, szs, emb_p, ps):
@@ -142,67 +199,13 @@ class TabularNet(nn.Module):
         x = self.fcs(x)
         return x
 
+################################################################################################################################
+#  Etc
+################################################################################################################################
 
-class SignalNet(nn.Module):
-    def __init__(self, data, p=0.):
-        super(SignalNet, self).__init__()
-        x,_ = data.ds1[0]
-        n1, sz = x.shape[0], x.shape[1]
-        n2 = min((n1+1)*8,64)
-        net = []
-        szs = [(n1,sz)]
-        for i in range(3):
-            net += [Conv1D(n1, n2, stride=2)]
-            n1, sz = n2, int(sz/2)
-            szs += [(n1,sz)]
-        bottle = int((int(math.log2(sz)))/2)
-        for i in range(bottle):
-            net += [ResBlock1D(n1, n1*2, ks=3, stride=2)]
-            n1, sz = n1*2, int(sz/2)
-            szs += [(n1,sz)]
-            net += [nn.MaxPool1d(2)]  # AvgPool1d, MaxPool1d
-            sz = int(sz/2)
-            szs += [(n1,sz)]
-        net += [nn.AdaptiveMaxPool1d(1)]
-        net += [Flatten()]
-        net += [Linear(1*n1, 1024, p=p)]
-        net += [LinearFinal(1024, data.c, p=None)]
-        self.net = nn.Sequential(*net)
-        p = sum([p.numel() for p in self.parameters()])
-        print(f'SignalNet: params: {p:,.0f}, signal_sz: {szs[0]}, bottles: {bottle}, last_conv {szs[-1]}')
-        # print(szs)
-
-    def forward(self, x): return self.net(x)
-
-
-class Resnet(nn.Module):
-    def __init__(self, data, layers=[64, 64, 64], p=None):
-        super(Resnet, self).__init__()
-        sz = [int(e) for e in data.ds1[0][0].shape]
-        n1, n2 = sz[0], 32
-        net = [ResBlock2D(n1, n2, stride=2)]
-        n1 = n2
-        for n2 in layers:
-            net += [ResBlock2D(n1, n2, stride=2)]
-            n1 = n2
-        net += [nn.AdaptiveMaxPool2d(1)]
-        net += [Flatten()]
-        net += [Linear(layers[-1], 1024, p=p)]
-        net += [LinearFinal(1024, data.c, p=None)]
-        self.net = nn.Sequential(*net)
-        p = sum([p.numel() for p in self.parameters()])
-        self.szs = get_cnn_szs(self, sz)
-        print(f'\nResnet: params: {p:,.0f}, img_sz: {sz}, last_conv {self.szs[-1]}: ')
-
-    def forward(self, x): return self.net(x)
-
-
-
-
-
-
-
-
+# x = torch.randn(1, 1, 250)
+# l = ResBlock1D(1, 64)
+# l(x).shape
 
 # class ResBlock(nn.Module):
 #     def __init__(self, expansion, ni, nh, stride=1):
